package {
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxPoint;
	import org.flixel.FlxSprite;
	import org.flixel.FlxText;

	public class Character extends FlxGroup {

		[Embed(source="../data/chatbubble.png")] 
		private var ImgBubble:Class;

		[Embed(source="../data/character.png")] 
		private var ImgCharacter:Class;

		public var _characterSprite:FlxSprite;
		private var _bubbleSprite:FlxSprite;
		private var _nameTag:FlxText;
		private var _statusText:FlxText;
		private var _speechText:FlxText;
		private var _name:String;
		private var _speechHeight:int;
		private var _cutscene:Array;

		public function Character(X:Number = 0, Y:Number = 0, Text:String = null, SimpleGraphic:Class = null):void {
			super();
			
			x = X;
			y = Y;
						
			// Character Sprite
			_characterSprite = new FlxSprite();
			_characterSprite.width = 16;
			_characterSprite.height = 16;
			_characterSprite.solid = true;
			
			_characterSprite.loadGraphic(ImgCharacter, true, true, 16, 16);
			_characterSprite.addAnimation("side", [2]);
			_characterSprite.play("side");
			add(_characterSprite);
			
			_name = Text;
			
			// Name tag
			_nameTag = new FlxText(-8, -16, 32, _name);
			_nameTag.alignment = "center";
			_nameTag.size = 8;
			_nameTag.width = 32;
			_nameTag.height = 32;
			_nameTag.solid = true;
			add(_nameTag);
			
			// Chat bubbles
			_bubbleSprite = new FlxSprite(16, -64);
			_bubbleSprite.width = 64;
			_bubbleSprite.height = 64;
			_bubbleSprite.loadGraphic(ImgBubble);
			_bubbleSprite.x = 16;
			_bubbleSprite.y = -64;
			_bubbleSprite.visible = false;
			add(_bubbleSprite); 
			
			// Status
			_statusText = new FlxText(-64, 0, 64);
			_statusText.text = "name: " + _name + "\n" + "morale: normal";
			_statusText.visible = false;
			add(_statusText);
			
			// Chat Speech
			_speechText = new FlxText(_bubbleSprite.x + 5, _bubbleSprite.y + 8, 60, " ");
			_speechText.alignment = "left";
			_speechText.size = 8;
			add(_speechText);
			
			_cutscene = new Array();
			
			FlxG.mouse.show();	
		}

		public function appendSpeech(Speech:String):void {
			_cutscene.push(Speech);
			//_speechText.text = Speech;
			//_speechHeight = Speech.length/_speechText.width;
			//_bubbleSprite.scale = new FlxPoint(1, _speechHeight);	
		}

		private function processSpeech():void {
			if(_cutscene.length > 0) {
				_bubbleSprite.visible = true;
				_speechText.text = _name + ":\n" + _cutscene.shift();
				
				// Reset the text and the bubble every time
				_bubbleSprite.scale = new FlxPoint(1, 1);
				_bubbleSprite.x = _characterSprite.x + 16;
				_bubbleSprite.y = _characterSprite.y - 64;
				_speechText.y = _bubbleSprite.y + 8;
				
				var bubbleTextHeight:int = _bubbleSprite.height - 10;
				
				if(_speechText.height + 16 > bubbleTextHeight) {
					// Rescale the height of the speech text
					var heightScale:Number = (_speechText.height + 16) / bubbleTextHeight;
					_bubbleSprite.scale = new FlxPoint(1, heightScale);
					
					// Scaling doesn't change the height of _bubbleSprite
					var newHeight:Number = heightScale * _bubbleSprite.height;

					// Adjust the y location of the scaled bubble
					var offset_y:int = (newHeight - _bubbleSprite.height) / 2;
					_bubbleSprite.y -= offset_y;
					
					// Adjust the y location of the text
					_speechText.y -= 2 * offset_y;
				}
			} else {
				_bubbleSprite.visible = false;
				_speechText.text = "";
				return;
			}
		}

		override public function update():void {
			super.update();
			
			// Movement
			/* velocity.x = 0;
			velocity.y = 0;
			
			if(FlxG.keys.LEFT)
			{
			velocity.x -= 100;
			}
			if(FlxG.keys.RIGHT)
			{
			velocity.x += 100;
			}
			if(FlxG.keys.UP)
			{
			velocity.y -= 100;
			}
			if(FlxG.keys.DOWN)
			{
			velocity.y += 100;
			} */
			if(_characterSprite.overlaps(FlxG.mouse.cursor)) {
				_statusText.visible = true;
			} else {
				_statusText.visible = false;
			}
			
			if(FlxG.mouse.justPressed()) {
				processSpeech();
			}
		}
	}
}