package {
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxObject;
	import org.flixel.FlxPoint;
	import org.flixel.FlxSprite;
	import org.flixel.FlxU;

	import flash.geom.Point;

	//This is the class declaration for the ship you fly
	public class Ship extends FlxGroup {
		// CONSTANTS
		
		/*
		 * Bullet speed
		 */
		private static const BULLET_SPEED:Number = 300;
		/*
		 * Fire interval for bullets
		 */
		private static const BULLET_INTERVAL:Number = 0.1;
		/*
		 * Maximum amount of energy the ship can hold
		 */
		private static const MAX_ENERGY:Number = 100;
		/*
		 * How much energy each bullet uses
		 */
		private static const BULLET_ENERGY_USE:Number = 5;
		/*
		 * Energy gain interval
		 */
		private static const ENERGY_INTERVAL:Number = 0.25;
		/*
		 * Energy gain per interval period
		 */
		private static const ENERGY_REGEN:Number = 4;
		/*
		 * Minimum energy needed to keep shielded
		 */
		private static const MIN_ENERGY_SHIELD:Number = 20;
		/*
		 * Amount of energy used to absorb 1 damage point to the shield
		 */
		private static const DAMAGE_TO_ENERGY_RATIO:Number = 1;
		/*
		 * Thrust
		 */
		private static const MAX_THRUST:Number = 150;
		/*
		 * Turn Speed
		 */
		private static const TURN_SPEED:Number = 240;

		// VARIABLES
		
		/*
		 * The backing variable for shipCore
		 */
		private var _shipCore:ShipPart;
		/*
		 * Sprite used to display the energy meter
		 */
		private var _energyMeter:ShipPart;
		/*
		 * Chat bubble for displaying "ouch" messages
		 */
		private var _chatBubble:ChatBubble;
		/*
		 * FlxGroup for storing list of bullets
		 */
		private var _bullets:FlxGroup;

		/*
		 * Index of next available bullet
		 */
		private var _bulletIndex:uint;
		/*
		 * Timer used for counting down until the next bullet can be fired
		 */
		private var _bulletTimer:Number;
		/*
		 * The backing variable for energy
		 */
		private var _energy:Number;
		/*
		 * Timer used for energy regeneration
		 */
		private var _energyTimer:Number;
		/*
		 * Determines whether or not the ship is shielded
		 */
		private var _shielded:Boolean;
		/*
		 * Point used to store previous location
		 */
		private var _preCollisionLoc:FlxPoint = new FlxPoint;
		/*
		 * Used to store the previous angle to check for changes.
		 */
		private var _prevAngle:Number;
		/*
		 * A vector representing the parts of the group responsible for collision
		 * Typically only the display portion of the ship is left out
		 */
		private var _colParts:Array;

		
		// TEMPORARY VARIABLES
		/*
		 * Temporary point used to conserve memory
		 */
		private var _tmp:FlxPoint = new FlxPoint();
		private var _tmp2:Point = new Point();

		// EMBEDDED RESOURCES

		/*
		 * Ship Graphic
		 */
		[Embed(source="../data/ship.png")] 
		private var ImgShip:Class;

		/*
		 * Energy Meter Graphic
		 */
		[Embed(source="../data/energy_meter.png")] 
		private var ImgEnergyMeter:Class;

		//This function creates the ship
		public function Ship(initX:Number, initY:Number,
			       bullets:FlxGroup) {
			super();

			//Initialize position
			this.x = initX;
			this.y = initY;
			width = 0;
			height = 0;
			
			// Set starting angle to point upwards, energy, and maximum thrust
			_prevAngle = angle = -90;
			maxThrust = MAX_THRUST;
			//drag.x = drag.y = maxThrust;
			_energy = MAX_ENERGY;
			_energyTimer = ENERGY_INTERVAL;
			
			// Initialize Collision Parts
			_colParts = new Array(10);
			
			//REMOVE LATER
			var c:uint = 0x0000ffff;
			
			var s:FlxSprite;
			//Top
			s = new ShipPart(this, -22, -50);
			s.createGraphic(45, 45, c);
			_colParts[0] = ShipPart(add(s));
			//Middle
			s = new ShipPart(this, -25, -25);
			s.createGraphic(50, 50, c);
			_colParts[1] = ShipPart(add(s));
			//Right 1
			s = new ShipPart(this, 25, -20);
			s.createGraphic(25, 25, c);
			_colParts[2] = ShipPart(add(s));
			//Right 2
			s = new ShipPart(this, 52, -20);
			s.createGraphic(25, 25, c);
			_colParts[3] = ShipPart(add(s));
			//Left 1
			s = new ShipPart(this, -77, -20);
			s.createGraphic(25, 25, c);
			_colParts[4] = ShipPart(add(s));
			//Left 2
			s = new ShipPart(this, -50, -20);
			s.createGraphic(25, 25, c);
			_colParts[5] = ShipPart(add(s));
			//Bottom 1
			s = new ShipPart(this, -18, 25);
			s.createGraphic(25, 25, c);
			_colParts[6] = ShipPart(add(s));
			//Bottom 2
			s = new ShipPart(this, -3, 25);
			s.createGraphic(25, 25, c);
			_colParts[7] = ShipPart(add(s));

			// Initialize Ship Core with shielded and unshielded animations
			_colParts[8] = add(_shipCore = new ShipPart(this, -56, -81));
			_shipCore.loadGraphic(ImgShip, true, false, 110, 162);
			_shipCore.addAnimation("shielded", [0], 0);
			_shipCore.addAnimation("unshielded", [1], 0);
			_shipCore.play("shielded");
			_shipCore.solid = false;
			_shipCore.moves = false;
			
			// Initialize Energy Meter
			_colParts[9] = add(_energyMeter = new ShipPart(this, -2, -90));
			_energyMeter.loadGraphic(ImgEnergyMeter);
			_energyMeter.solid = false;
			_energyMeter.moves = false;
			
			// Initialize Chat Bubble
			_chatBubble = new ChatBubble(_shipCore);
			add(_chatBubble);
			
			// Initialize Bullets Group
			_bullets = bullets;
			//			_bullets = new FlxGroup(); // TEMPORARY, should be an array passed through the constructor by spaceview
			
			// Bullets
			_bulletTimer = BULLET_INTERVAL;
			for (var i:int = 0;i < 50;i++) {
				s = new FlxSprite(-100, -100);
				s.createGraphic(5, 5);
				_bullets.add(s);
			}
			PlayState(FlxG.state).add(_bullets);
		}

		//The main game loop function
		override public function update():void {
			//Timers
			_bulletTimer -= FlxG.elapsed;
			_energyTimer -= FlxG.elapsed;
			
			//This is where we handle energy regen
			//Use a while loop since the regen time is so small
			while(_energyTimer < 0 && _energy < MAX_ENERGY) {
				modifyEnergy(ENERGY_REGEN);
				_energyTimer += ENERGY_INTERVAL;
			}
			
			//Handles shielding
			_shielded = _energy > MIN_ENERGY_SHIELD;
			if(_shielded)
				_shipCore.play("shielded");
			else
				_shipCore.play("unshielded");
			
			//This is where we handle turning the ship left and right
			angularVelocity = 0;
			if(FlxG.keys.LEFT)
      			angularVelocity -= TURN_SPEED;
			if(FlxG.keys.RIGHT)
      			angularVelocity += TURN_SPEED;
			
			//This is where thrust is handled
			thrust = 0;
			if(FlxG.keys.UP)
      			thrust -= maxThrust * 3;
			if(FlxG.keys.DOWN)
      			thrust += maxThrust * 3;

			//Store previous location and angle
			_preCollisionLoc.x = x;
			_preCollisionLoc.y = y;
			_prevAngle = angle;
			
			//The all important parent game loop update - this calls FlxGroup.update()
			super.update();

			//If angle has been updated, rotate all members 
			if(_prevAngle != angle) {
				rotateMembers(angle - _prevAngle);
			}

			//Set the core's angle to the angle of the group so it displays properly
			_shipCore.angle = _energyMeter.angle = angle;

			//Firing Bullets
			if(_bulletTimer <= 0 && _energy >= BULLET_ENERGY_USE && FlxG.keys.Z) {
				//Z was pressed!  FIRE A BULLET
				var b:FlxSprite = _bullets.members[_bulletIndex];	//Figure out which bullet to fire
				if(_bulletIndex % 2 == 0) {
					b.reset(ShipPart(_colParts[3]).x + (ShipPart(_colParts[3]).width - b.width) / 2, ShipPart(_colParts[3]).y + (ShipPart(_colParts[3]).height - b.height) / 2);
					b.angle = angle - 10;
				} else {
					b.reset(ShipPart(_colParts[4]).x + (ShipPart(_colParts[4]).width - b.width) / 2, ShipPart(_colParts[4]).y + (ShipPart(_colParts[4]).height - b.height) / 2);
					b.angle = angle + 10;
				}
				b.velocity = FlxU.rotatePoint(BULLET_SPEED, 0, 0, 0, b.angle);
				b.velocity.x += velocity.x;
				b.velocity.y += velocity.y;
				_bulletIndex++; //Increment our bullet list tracker
				if(_bulletIndex >= _bullets.members.length) //And check to see if we went over
					_bulletIndex = 0; //If we did just reset.
				_bulletTimer = BULLET_INTERVAL; //Reset bullet timer to interval
				modifyEnergy(-BULLET_ENERGY_USE);
			}
		}

		/*
		 * Used to move the entire ship group to resolve a collision with any of its parts.
		 * Called by overlap(_ship, ..., _ship.resolveCollision )
		 */
		public function resolveCollision(SPart:FlxObject, ColObj:FlxObject, Damage:Number = 0):void {
			// Apply damage if necessary
			ShipPart(SPart).hurt(Damage);
			
			// Reverse previous rotation 
			_tmp.x = angle;
			angle = _prevAngle;
			_prevAngle = _tmp.x;
			
			rotateMembers(angle - _prevAngle);
			angularVelocity *= -1;

			if(!ColObj.fixed) {
				// Push any objects we collide with away from us
				var speed:Number;
				// Calculate Obj Speed
				_tmp2.x = ColObj.velocity.x;
				_tmp2.y = ColObj.velocity.y;
				speed = _tmp2.length;
				// Calculate our Speed
				_tmp2.x = velocity.x;
				_tmp2.y = velocity.y;
				speed += _tmp2.length;
				// Set bounce off speed of asteroid
				_tmp2.x = ColObj.x - x;
				_tmp2.y = ColObj.y - y;
				_tmp2.normalize(speed / 2);
				ColObj.velocity.x = _tmp2.x;
				ColObj.velocity.y = _tmp2.y;
			}

			// If after reversing rotation there is still a collision, reset position
			if(SPart.overlaps(ColObj))
				reset(_preCollisionLoc.x, _preCollisionLoc.y);
				
			// Check if continued motion in this direction will cause a collision
			// and reverse velocity as necessary
			var off:Number;
			
			if(velocity.x < 0)
				off = -3;
			else
				off = 3;
			SPart.x += off;
			if(SPart.overlaps(ColObj)) {
				velocity.x *= -0.8;
			}
			SPart.x -= off;				

			if(velocity.y < 0)
				off = -3;
			else
				off = 3;
			SPart.y += off;
			if(SPart.overlaps(ColObj)) {
				velocity.y *= -0.8;
			}
			SPart.y -= off;
		}

		/*
		 * Pass through for showing a chat bubble message coming out of the ship
		 */
		public function showMessage(Text:String, Duration:Number):void {
			_chatBubble.showSpeech(Text, Duration);
		}

		/*
		 * Utility function used to rotate all members of the group by their origin
		 * around the x,y location of the group
		 */
		private function rotateMembers(deltaAngle:Number):void {
			var len:int = _colParts.length;
			for (var i:int = 0;i < len;i++) {
				if(_colParts[i] != _shipCore) {
					_tmp = FlxU.rotatePoint(ShipPart(_colParts[i]).x + ShipPart(_colParts[i]).origin.x, ShipPart(_colParts[i]).y + ShipPart(_colParts[i]).origin.y, x, y, deltaAngle);
					ShipPart(_colParts[i]).x = _tmp.x - ShipPart(_colParts[i]).origin.x;
					ShipPart(_colParts[i]).y = _tmp.y - ShipPart(_colParts[i]).origin.y;
				}
			}
		}

		/*
		 * Utility function for modifying the energy of the ship by a delta value
		 * and checking min/max values.
		 */
		private function modifyEnergy(deltaEnergy:Number):void {
			_energy += deltaEnergy;
			
			if(_energy < 0) {
				_energy = 0;
				_energyMeter.scale.x = 0.25;
				_energyMeter.scale.y = 0;
				return;
			}
			if(_energy > MAX_ENERGY) {
				_energy = MAX_ENERGY;
				_energyMeter.scale.x = 1;
				_energyMeter.scale.y = 1;
				return;
			}
			_tmp.x = (_energy / MAX_ENERGY);
			_energyMeter.scale.x = 0.25 + _tmp.x * 0.75;
			_energyMeter.scale.y = _tmp.x;
		}

		/*
		 * Function to take care of hurting the shield
		 * ShipPart calls this function in its hurt to
		 * check if the shield absorbs the damage or the part
		 * 
		 * @return How much damage gets through to the part
		 */
		public function hurtShield(Damage:Number):Number {
			// If not shielded, all damage goes to the shipPart
			if(_shielded ) {
				// Calculate energy used by damage absorbtion
				var energyUse:Number = Damage * DAMAGE_TO_ENERGY_RATIO;
				if(_energy > energyUse) {
					// Remove energy and tell the part that it takes no damage
					modifyEnergy(-energyUse);
					return 0;
				} else {
					// Reduce damage equal to energy used divided by ratio
					Damage -= _energy / DAMAGE_TO_ENERGY_RATIO;
					modifyEnergy(-_energy);
					return Damage;
				}
			} else {
				return Damage;
			}
		}

		/*
		 * Variable used to store the ship's core and display object
		 * This object is used for all typical FlxSprite functions
		 * like follow bounds
		 * Getter function provided
		 */
		public function get shipCore():ShipPart {
			return _shipCore;
		}

		/*
		 * TEMPORARY Getter function for getting the bullets group
		 */
		public function get bullets():FlxGroup {
			return _bullets;
		}

		/*
		 * How much energy the ship currently has
		 */
		public function get energy():Number {
			return _energy;
		}
	}
}