package {
	import org.flixel.FlxSprite;

	/**
	 * @author Yori Kvitchko
	 * 
	 * Class responsible for representing a part of the composite ship object
	 */
	public class ShipPart extends FlxSprite {
		private var _parent:Ship;
		/*
		 * Messages shown when hurt
		 */
		private var _hurtMessages:Array = ["Ouch!", "Oof.", "Hey! Stop that!"];

		public function ShipPart(Parent:Ship, X:Number = 0, Y:Number = 0, SimpleGraphic:Class = null) {
			super(X, Y, SimpleGraphic);
			this.health = 1;
			this._parent = Parent;
		}

		public function get parent():Ship {
			return _parent;
		}

		/*
		 * First passes the damage through the parent ship's
		 * shields and then damages the individual part if
		 * any damage is left after shields
		 */
		override public function hurt(Damage:Number):void {
			Damage = _parent.hurtShield(Damage);
			
			if(Damage > 0)
				super.hurt(Damage);
		}

		/*
		 * Don't kill, instead just show a random message
		 */
		override public function kill():void {
			_parent.showMessage(_hurtMessages[Math.floor(Math.random() * _hurtMessages.length)], 1);
		}
	}
}
