package {
import org.flixel.*;

public class SpaceView extends FlxGroup {
  // Space MAP dimensions [TEMP]
  public const W:Number = 4*640, H:Number = 1.5*480;

  // Size of individual tile (for use in blocks, etc) [TEMP]
  private var tSize:Number = 16;

    // Image for goal galaxy
[Embed(source='../data/galaxy.png')]
  private var ImgGoal:Class;

[Embed(source='../data/star.png')]
  private var ImgStar:Class;

  // small stars
[Embed(source='../data/bgtile.png')]
  private const ImgBG:Class;

  // big stars
[Embed(source='../data/bgtile2.png')]
  private const ImgBG2:Class;

  private var _bg:FlxTileblock;

  public var _border:FlxGroup;
  public var _enemies:FlxGroup;
  public var _bullets:FlxGroup;
  public var _objects:FlxGroup;
  public var _ship:Ship;
  public var _goal:FlxSprite;
  public var _radar:Radar;

  public var emitter:FlxEmitter;
  
  public function SpaceView():void {
    super();

    initBG();
    
    _enemies = new FlxGroup();
    /*
    _enemies.add(new FlxSprite(150, 150));
    FlxSprite(_enemies.members[0]).createGraphic(500, 20);
    add(_enemies);
    */

    emitter = new FlxEmitter(-100, -100);
    emitter.createSprites(ImgStar, 7);
    emitter.gravity = 0;
    emitter.setXSpeed(-500,500);
    emitter.setYSpeed(-500,500);
    add(emitter);
    
    add(createGoal(.95*W, H/2));
    
    add(createObjects(30));
    
    add(createBorder());
	
    _bullets = new FlxGroup;
    _ship = new Ship(100, H/2, _bullets);
    add(_ship);
    
    FlxG.follow(_ship.shipCore, 2.5);
    FlxG.followAdjust(0.5, 0.0);
    FlxG.followBounds(0, 0, W, H);

    _radar = new Radar(this);
    add(_radar);
  }

  private function initBG():void {
    addStarGroup(ImgBG,.1,140);
    addStarGroup(ImgBG2,.2,140);
    addStarGroup(ImgBG,.3,140);
    addStarGroup(ImgBG2,.4,140);
    addStarGroup(ImgBG,.5,140);
    addStarGroup(ImgBG2,.6,140);
  }


  private function createGoal(x:Number=0, y:Number=0):FlxSprite {
    _goal = new FlxSprite(x,y,ImgGoal);
    _goal.angularVelocity = -5;
    return _goal;
  }
				
  
  private function createBorder(x:Number=0, y:Number=0,
				w:Number=W, h:Number=H):FlxGroup {
    _border = new FlxGroup();
    _border.visible = false;
    // Ceiling
    _border.add(new FlxTileblock(x, y, w, 1));
    // Floor
    _border.add(new FlxTileblock(x, h-1, w, 1));
    // Left
    _border.add(new FlxTileblock(x, y+1, 1, h-2));
    // Right
    _border.add(new FlxTileblock(w-1, y+1, 1, h-2));
    return _border;
  }
  
  private function addStarGroup(Img:Class, scroll:Number=0,
				e:Number=100):void {
    var _bg:FlxTileblock = new FlxTileblock(0, 0, W, H);
    _bg.loadGraphic(Img, e);
    _bg.scrollFactor = new FlxPoint(scroll, scroll);
    _bg.solid = false;
    add(_bg);
  }

  private function createObjects(n:uint=10):FlxGroup {
    var V:Number = 50;

    _objects = new FlxGroup;


    var a:Asteroid = new Asteroid();
    _objects.add(new Rock().create(300, 65));
    _objects.add(new Rock().create(400, 250));


    _objects.add(new Rock().create(700, 40));
    
    _objects.add(new Rock().create(680, 600));
    _objects.add(new Rock().create(770, 500));
    _objects.add(new Rock().create(880, 450));

    _objects.add(new Rock().create(1260, 200));
    _objects.add(new Rock().create(1320, 300));

    _objects.add(new Rock().create(1500, 260));
    _objects.add(new Rock().create(1650, 200));
    _objects.add(new Rock().create(1750, 70));

    _objects.add(new Rock().create(2000, 500));
    _objects.add(new Rock().create(2150, 420));
    _objects.add(new Rock().create(2250, 600));
        
    for (var i:uint = 1; i <= n; i++) {
      
      _objects.add(new Asteroid().create(200+.8*W*Math.random(), H*Math.random(),
					 V*Math.random(), V*Math.random(),
					 a.ImgMedium));
      
      _objects.add(new Asteroid().create(200+.8*W*Math.random(), H*Math.random(),
					 V*Math.random(), V*Math.random(),
					 a.ImgSmall));
    }

    /*
    for each (var o:FlxSprite in _objects.members) {
	o.scale.x=o.scale.y=3;
      }
    */

    return _objects;
  }

  private function win(SPart:FlxObject, ColObj:FlxObject):void {
    FlxG.state = new MenuState;
  }

  
  private function shot(bullet:FlxObject, obj:FlxObject):void {
    if (!obj.fixed && !bullet.fixed) {
      emitter.at(obj);
      emitter.start(true, 1);

      bullet.kill();
      obj.kill();
    }
  }
  
  override public function update():void {
    super.update();
			
    //collisions with environment
	FlxU.overlap(_ship, _enemies, shipBulletCollision);
	FlxU.overlap(_ship, _objects, shipAsteroidCollision);
	FlxU.overlap(_ship, _border, shipBorderCollision);
    FlxU.overlap(_ship, _goal, win);
    FlxU.overlap(_bullets, _objects, shot);
    
    FlxU.collide(_objects, _border);
    FlxU.collide(_objects, _objects);
  }

		/*
		 * Triggered by a collision between ship and an asteroid
		 */
		private function shipAsteroidCollision(Obj1:FlxObject, Obj2:FlxObject):void {
			if(Obj1 is ShipPart)
				_ship.resolveCollision(Obj1, Obj2, 10);
			else
				_ship.resolveCollision(Obj2, Obj1, 10);
		}

		/*
		 * Triggered by a collision between ship and a bullet
		 */
		private function shipBulletCollision(Obj1:FlxObject, Obj2:FlxObject):void {
			if(Obj1 is ShipPart)
				_ship.resolveCollision(Obj1, Obj2, 20);
			else
				_ship.resolveCollision(Obj2, Obj1, 20);
		}

		/*
		 * Triggered by a collision between ship and the map border
		 */
		private function shipBorderCollision(Obj1:FlxObject, Obj2:FlxObject):void {
			if(Obj1 is ShipPart)
				_ship.resolveCollision(Obj1, Obj2);
			else
				_ship.resolveCollision(Obj2, Obj1);
		}
		
  /*
   * Triggered by PlayState to indicate a view switch
   */
  public function switchView(inside:Boolean):void {
    if(!inside) {
      FlxG.mouse.hide();
      FlxG.follow(_ship.shipCore, 2.5);
    }
  }
}
}